'''DNA Replicator

You are a molecule of DNA Replicase.  Your mission, replicate DNA.
You must have the timing to add the correct base at the correct
time.  (This game is cheap knock-off of Dance Dance Revolution.)

'''

from __future__ import division

import sys
import optparse
import random
import array

import pygame
from OpenGL.GL import *

import data

SCREENWIDTH = 640
SCREENHEIGHT = 480

compliment = {
    'adenine': 'thymine',
    'thymine': 'adenine',
    'cytosine': 'guanine',
    'guanine': 'cytosine',
    }

basex = {
    'adenine': 3.0,
    'cytosine': 13.0,
    'guanine': 23.0,
    'thymine': 33.0,
    }


class Message(object):
    def __init__(self,x,y,text,font,color,expiration):
        surf = font.render(text,True,color)
        width,height = surf.get_size()
        for j in (4,8,16,32,64,128,256,512):
            if j >= width:
                uwidth = j
                break
        else:
            width = uwidth = 512
        for j in (4,8,16,32,64,128,256,512):
            if j >= height:
                uheight = j
                break
        else:
            height = uheight = 512
        upsurf = pygame.Surface((uwidth,uheight),pygame.SRCALPHA,32)
        upsurf.set_alpha(0.0)
        upsurf.blit(surf,(0,uheight-height))
        texdata = pygame.image.tostring(upsurf,"RGBA",True)
        self.tex = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D,self.tex)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST)
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,uwidth,uheight,0,
                     GL_RGBA, GL_UNSIGNED_BYTE,texdata)
        self.x0 = x - 100.0/SCREENWIDTH*width/2
        self.y0 = y - 75.0/SCREENHEIGHT*height/2
        self.x1 = self.x0 + 100.0/SCREENWIDTH*width
        self.y1 = self.y0 + 75.0/SCREENHEIGHT*height
        self.tw = width/uwidth
        self.th = height/uheight
        self.expiration = expiration
    def draw(self):
        glBindTexture(GL_TEXTURE_2D,self.tex)
        glEnable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)
        glColor4f(1.0,1.0,1.0,1.0)
        glTexCoord2f(0.0,0.0)
        glVertex3f(self.x0,self.y0,-1.0)
        glTexCoord2f(self.tw,0.0)
        glVertex3f(self.x1,self.y0,-1.0)
        glTexCoord2f(self.tw,self.th)
        glVertex3f(self.x1,self.y1,-1.0)
        glTexCoord2f(0.0,self.th)
        glVertex3f(self.x0,self.y1,-1.0)
        glEnd()
        glDisable(GL_TEXTURE_2D)
    def cleanup(self):
        glDeleteTextures((self.tex,))

def compilemesh(a):
    dl = glGenLists(1)
    if dl == 0:
        raise GLError("cannot allocate display list")
    glNewList(dl,GL_COMPILE)
    index = 0
    while index < len(a):
        npoints = int(a[index])
        index += 1
        color = list(a[index:index+3])
        color.append(1.0)
        index += 3
        glMaterialfv(GL_FRONT,GL_AMBIENT,color)
        glMaterialfv(GL_FRONT,GL_DIFFUSE,color)
        glMaterialfv(GL_FRONT,GL_SPECULAR,(1.0,1.0,1.0,1.0))
        glMaterialf(GL_FRONT,GL_SHININESS,0.0)
        glBegin(GL_TRIANGLES)
        for i in xrange(3*npoints):
            glNormal3f(*a[index:index+3])
            index += 3
            glVertex3f(*a[index:index+3])
            index += 3
        glEnd()
    glEndList()
    return dl

def main():
    parser = optparse.OptionParser(usage = __doc__,version = 0.1)
    parser.add_option("-f","--fullscreen",
                      action = "store_true",
                      default = False,
                      help = "Play in fullscreen mode")
    options,args = parser.parse_args()
    if len(args) != 0:
        print >> sys.stderr, "No position args allowed for this game"
        sys.exit(1)

    flags = pygame.OPENGL|pygame.DOUBLEBUF
    if options.fullscreen:
        flags |= pygame.FULLSCREEN

    pygame.init()
    pygame.display.set_mode((SCREENWIDTH,SCREENHEIGHT),flags)
    pygame.display.set_caption("DNA Replicator")
    pygame.mouse.set_pos(SCREENWIDTH//2,SCREENHEIGHT//2)
    pygame.mouse.set_visible(False)

    global basedl
    basedl = {}
    for base in ('adenine','guanine','cytosine','thymine'):
        a = array.array('f')
        if sys.byteorder == 'big':
            a.byteswap()
        flo = open(data.filepath('%s.msh' % base),'rb')
        try:
            a.fromstring(flo.read())
        finally:
            flo.close()
        basedl[base] = compilemesh(a)

    global txfont
    txfont = pygame.font.Font(data.filepath("FreeSansBold.ttf"),14)

    global fbfont
    fbfont = pygame.font.Font(data.filepath("FreeSansBold.ttf"),20)

    global bnfont
    bnfont = pygame.font.Font(data.filepath("FreeSansBold.ttf"),50)

    global opfont
    opfont = pygame.font.Font(data.filepath("FreeSansBold.ttf"),30)

    global ttfont
    ttfont = pygame.font.Font(data.filepath("FreeSerifBold.ttf"),50)

    glEnable(GL_DEPTH_TEST)
    glEnable(GL_BLEND)
    glEnable(GL_CULL_FACE)
    glEnable(GL_LIGHTING)
    glEnable(GL_LIGHT0)
    glEnable(GL_NORMALIZE)
    glShadeModel(GL_SMOOTH)
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
    glClearColor(0.5,0.5,1.0,0.0)
    glLightfv(GL_LIGHT0,GL_AMBIENT,(0.5,0.5,0.5,1.0))
    glLightfv(GL_LIGHT0,GL_DIFFUSE,(0.5,0.5,0.5,1.0))
    glLightfv(GL_LIGHT0,GL_SPECULAR,(0.0,0.0,0.0,1.0))
    glLightfv(GL_LIGHT0,GL_POSITION,(1.0,1.0,1.0,0.0))

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(0.0,100.0,0.0,75.0,0.0,100.0)
    glMatrixMode(GL_MODELVIEW)

    global clock
    clock = pygame.time.Clock()

    option = 0
    while True:
        option = titlescreen(option)

        if option == 0:
            nbases = 35
            mingaptime = 25
            maxgaptime = 51
            leadtime = 80
            tutorial = True
        elif option == 1:
            nbases = 50
            mingaptime = 15
            maxgaptime = 31
            leadtime = 60
            tutorial = False
        elif option == 2:
            nbases = 60
            mingaptime = 9
            maxgaptime = 22
            leadtime = 50
            tutorial = False
        elif option == 3:
            nbases = 70
            mingaptime = 7
            maxgaptime = 15
            leadtime = 40
            tutorial = False
        elif option == 4:
            nbases = 80
            mingaptime = 5
            maxgaptime = 11
            leadtime = 30
            tutorial = False

        if tutorial:
            status = intro()
            if not status:
                continue

        score,hits = level(nbases,mingaptime,maxgaptime,leadtime,tutorial)
        if score == -2:
            continue

        scoreboard(score,hits,nbases)


def titlescreen(option):

    bases = [ random.choice(('adenine','guanine','cytosine','thymine'))
              for i in xrange(26) ]

    messages = (
        Message(50.0,60.0,"DNA REPLICATOR",ttfont,(0,0,0),0),
        Message(50.0,46.0,"Play Tutorial",opfont,(0,0,0),0),
        Message(50.0,38.0,"Play Easy Game",opfont,(0,0,0),0),
        Message(50.0,30.0,"Play Medium Game",opfont,(0,0,0),0),
        Message(50.0,22.0,"Play Hard Game",opfont,(0,0,0),0),
        Message(50.0,14.0,"Play Insane Game",opfont,(0,0,0),0),
        Message(50.0,5.5,"programmed by Carl Banks",txfont,(0,0,255),0),
        Message(50.0,3.0,"http://www.aerojockey.com/software/games/",
                txfont,(0,0,255),0),
        )

    timer = 0
    while True:

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                raise SystemExit
            elif event.type == pygame.KEYDOWN:
                if event.key in (pygame.K_q,pygame.K_ESCAPE):
                    raise SystemExit
                elif event.key in (pygame.K_k,pygame.K_UP):
                    if option > 0:
                        option -= 1
                elif event.key in (pygame.K_j,pygame.K_DOWN):
                    if option < 4:
                        option += 1
                elif event.key in (pygame.K_KP_ENTER,pygame.K_SPACE,
                                   pygame.K_RETURN):
                    for msg in messages:
                        msg.cleanup()
                    return option
                      
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()

        glPushMatrix()
        glTranslatef(-1.0,30.0,-70.0)
        glRotatef(-90.0,0.0,0.0,1.0)
        glRotatef(timer%360,0.0,1.0,0.0)
        for base in bases:
            glCallList(basedl[base])
            glPushMatrix()            
            glRotatef(180.0,0.0,0.0,1.0)
            glCallList(basedl[compliment[base]])
            glPopMatrix()
            glTranslatef(0.0,4.0,0.0)
            glRotatef(36.0,0.0,1.0,0.0)
        glPopMatrix()

        glDisable(GL_LIGHTING)
        glBegin(GL_QUADS)
        glColor4f(0.5,0.5,1.0,0.85)
        glVertex3f(0.0,45.0,-50.0)
        glVertex3f(0.0,15.0,-50.0)
        glVertex3f(100.0,15.0,-50.0)
        glVertex3f(100.0,45.0,-50.0)
        glEnd()

        for msg in messages:
            msg.draw()

        glEnable(GL_LIGHTING)
       
        glPushMatrix()
        glTranslatef(22.0,46.0-8.0*option,-20.0)
        glRotatef(180.0,0.0,0.0,1.0)
        glCallList(
            basedl[('adenine','guanine','cytosine','thymine')[option%4]])
        glPopMatrix()

        glPushMatrix()
        glTranslatef(77.0,46.0-8.0*option,-20.0)
        glCallList(
            basedl[('thymine','cytosine','guanine','adenine')[option%4]])
        glPopMatrix()

        timer += 1
        clock.tick(30)
        pygame.display.flip()



def intro():

    messages = set((
        Message(60.0,70.0,"Life:",fbfont,(255,255,255),9999),
        Message(70.0,55.0,(u"\u2190 Hit the appropriate key as the "
                           u"nucleotides pass."),txfont,(0,0,0),9999),
        Message(70.0,52.0,"(Middle row, right hand)",
                txfont,(0,0,0),9999),
        Message(5.0,55.0,"H",txfont,(0,0,0),9999),
        Message(15.0,55.0,"J",txfont,(0,0,0),9999),
        Message(25.0,55.0,"K",txfont,(0,0,0),9999),
        Message(35.0,55.0,"L",txfont,(0,0,0),9999),
        Message(65.0,40.0,"Your timing must be good.",txfont,(0,0,0),9999),
        Message(65.0,34.0,"You lose life if you fail to grab the nucleotide",
                txfont,(0,0,0),9999),
        Message(65.0,31.0,"at the right time, or if you hit the wrong key.",
                txfont,(0,0,0),9999),                
        Message(65.0,25.0,"Hitting the right key on time restores life.",
                txfont,(0,0,0),9999),
        Message(65.0,19.0,"in a regular game, if your life reaches zero:",
                txfont,(0,0,0),9999),
        Message(65.0,16.0,"the game is over, you lose.",
                txfont,(0,0,0),9999),
        Message(65.0,10.0,"Press RETURN when you are ready.",
                fbfont,(0,0,0),9999),
        ))

    while True:

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                raise SystemExit
            elif event.type == pygame.KEYDOWN:
                if event.key in (pygame.K_q,pygame.K_ESCAPE):
                    for msg in messages:
                        msg.cleanup()
                    return False
                elif event.key in (pygame.K_KP_ENTER,pygame.K_SPACE,
                                   pygame.K_RETURN):
                    for msg in messages:
                        msg.cleanup()
                    return True


        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()

        for base in ('adenine','guanine','cytosine','thymine'):
            glPushMatrix()
            glTranslatef(basex[base],55.0,-20.0)
            glCallList(basedl[base])
            glPopMatrix()

        glDisable(GL_LIGHTING)

        for msg in messages:
            msg.draw()

        glColor3f(0.0,1.0,0.0)
        glBegin(GL_QUADS)
        glVertex3f(66.0,69.0,-1.0)    
        glVertex3f(86.0,69.0,-1.0)    
        glVertex3f(86.0,71.0,-1.0)    
        glVertex3f(66.0,71.0,-1.0)
        glEnd()
        glColor3f(0.0,0.0,0.0)
        glBegin(GL_LINE_LOOP)
        glVertex3f(66.0,69.0,-0.99)    
        glVertex3f(86.0,69.0,-0.99)    
        glVertex3f(86.0,71.0,-0.99)    
        glVertex3f(66.0,71.0,-0.99)
        glEnd()

        glEnable(GL_LIGHTING)

        clock.tick(30)
        pygame.display.flip()



def level(nbases,mingaptime,maxgaptime,leadtime,tutorial):

    bases = []
    times = []
    scores = []
    totaltime = 0
    for i in xrange(nbases):
        bases.append(random.choice(('adenine','guanine','cytosine','thymine')))
        times.append(totaltime)
        scores.append(-1)
        totaltime += random.randrange(mingaptime,maxgaptime)

    riserindex = 0
    liveindex = 0
    templindex = 0

    maxlife = 2*nbases
    life = [maxlife]  ## GRRR.
    
    riserset = set()
    transset = set()
    templset = set()
    copyset = set()
    msgset = set((
        Message(60.0,70.0,"Life:",fbfont,(255,255,255),9999),
        Message(50.0,40.0,"Get Ready",bnfont,(0,255,0),0),
        Message(50.0,30.0,"Use VI keys (H,J,K,L) to add nucleotides",
                fbfont,(0,255,0),0),
        ))

    if tutorial:
        msgset.update((
            Message(5.0,55.0,"H",txfont,(0,0,0),9999),
            Message(15.0,55.0,"J",txfont,(0,0,0),9999),
            Message(25.0,55.0,"K",txfont,(0,0,0),9999),
            Message(35.0,55.0,"L",txfont,(0,0,0),9999),
            ))
        lifewarning = False

    autotap = None
    paused = False

    timer = -60
        
    endtime = times[-1] + leadtime + 100

    scoring = {
        3: 1,
        2: 2,
        1: 5,
        0: 10,
        }

    def tap(base):
        if liveindex > nbases:
            return
        if scores[liveindex] != -1:
            return
        diff = abs(times[liveindex]-timer+leadtime)
        if diff > 10:
            return
        msgy = 60.0
        if base != bases[liveindex]:
            msgx = basex[base]+2.5
            scores[liveindex] = 0
            msgset.add(
                Message(msgx,msgy,"Bzzt!",fbfont,(255,255,255),timer+15))
            life[0] = max(0,life[0]-10)
            return
        msgx = basex[bases[liveindex]]+2.5
        if diff > 3:
            scores[liveindex] = 0
            msgset.add(Message(msgx,msgy,"Miss",fbfont,(255,0,0),timer+15))
            life[0] = max(0,life[0]-10)
        else:
            scores[liveindex] = scoring[diff]
            riserset.remove(liveindex)
            transset.add(liveindex)
            if diff == 0:
                text = "Perfect!"
            else:
                text = "Hit!"
            msgset.add(Message(msgx,msgy,text,fbfont,(0,0,0),timer+15))
            life[0] = min(life[0]+scoring[diff],maxlife)

    while True:
        if not paused:
            if tutorial and life[0] < 33 and not lifewarning:
                msgset.add(Message(20.0,40.0,"Watch your life meter!",
                                   fbfont,(0,0,0),timer+90))
                
            if (liveindex < nbases-1
                and timer > (times[liveindex+1]+times[liveindex])//2+leadtime):
                if scores[liveindex] == -1:
                    scores[liveindex] = 0
                    msgy = 60.0
                    msgx = basex[bases[liveindex]]+2.5
                    msgset.add(
                        Message(msgx,msgy,"Missed",fbfont,(255,0,0),timer+15))
                    life[0] = max(0,life[0]-10)
                liveindex += 1

            if riserindex < nbases and timer >= times[riserindex]:
                riserset.add(riserindex)
                riserindex += 1

            while True:
                if templindex >= nbases:
                    break
                ty = (56.0 - templindex*4.0
                      + nbases*4.0/totaltime*(timer-leadtime))
                if ty < -3.0:
                    break
                templset.add(templindex)
                templindex += 1
           

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                raise SystemExit
            elif event.type == pygame.KEYDOWN:
                if event.key in (pygame.K_q,pygame.K_ESCAPE):
                    for msg in msgset:
                        msg.cleanup()
                    return -2,0
                if not paused:
                    if event.key in (pygame.K_h,pygame.K_LEFT):
                        tap("adenine")
                    elif event.key in (pygame.K_k,pygame.K_UP):
                        tap("guanine")
                    elif event.key in (pygame.K_j,pygame.K_DOWN):
                        tap("cytosine")
                    elif event.key in (pygame.K_l,pygame.K_RIGHT):
                        tap("thymine")
                if event.key == pygame.K_SPACE:
                    paused = not paused

        if timer >= endtime:
            if scores[-1] == -1:
                scores[-1] = 0
            assert -1 not in scores
            for msg in msgset:
                msg.cleanup()
            return sum(scores),nbases-scores.count(0)

        if not tutorial and life[0] <= 0:
            for msg in msgset:
                msg.cleanup()
            return -1,0
                       
        if (not paused
            and autotap is not None
            and liveindex < bases and timer == times[liveindex]+leadtime
            and random.random() < autotap):
            tap(bases[liveindex])

        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()

        for i in list(transset):
            tt = timer-times[i]-leadtime            
            if tt > 30:
                transset.remove(i)
                copyset.add(i)
                continue
            if tt < 5:
                continue
            tr = (tt-5)/25
            ty = 56.0 - i*4.0 + nbases*4.0/totaltime*(timer-leadtime)
            glPushMatrix()
            glTranslatef(basex[bases[i]]*(1.0-tr),55.0*(1.0-tr)+30.0*tr,-70.0)
            glRotatef(-90.0*tr,0.0,0.0,1.0)
            glRotatef((timer*3)%360*tr,0.0,1.0,0.0)
            glTranslatef(0.0,ty*tr,0.0)
            glRotatef(-(i%10)*36.0*tr,0.0,1.0,0.0)
            glRotatef(180.0*tr,0.0,0.0,1.0)
            glCallList(basedl[bases[i]])
            glPopMatrix()

        glPushMatrix()
        glTranslatef(0.0,30.0,-70.0)
        glRotatef(-90.0,0.0,0.0,1.0)
        glRotatef((timer*3)%360,0.0,1.0,0.0)
        for i in list(templset):
            ty = 56.0 - i*4.0 + nbases*4.0/totaltime*(timer-leadtime)
            if ty >= 103.0:
                templset.remove(i)
                copyset.discard(i)
                continue
            glPushMatrix()
            glTranslatef(0.0,ty,0.0)
            glRotatef(-(i%10)*36.0,0.0,1.0,0.0)
            glCallList(basedl[compliment[bases[i]]])
            if i in copyset:
                glRotatef(180.0,0.0,0.0,1.0)
                glCallList(basedl[bases[i]])
            glPopMatrix()
        glPopMatrix()

        glDisable(GL_LIGHTING)
        glBegin(GL_QUAD_STRIP)
        glColor4f(0.5,0.5,1.0,0.7)
        glVertex3f(0.0,45.0,-50.0)
        glVertex3f(0.0,15.0,-50.0)
        glVertex3f(40.0,45.0,-50.0)
        glVertex3f(40.0,15.0,-50.0)
        glColor4f(0.5,0.5,1.0,0.0)
        glVertex3f(50.0,45.0,-50.0)
        glVertex3f(50.0,15.0,-50.0)
        glEnd()
        glEnable(GL_LIGHTING)

        for i in list(riserset):
            ypos = (timer-times[i])/leadtime * 58.0 - 3.0
            if ypos >= 78.0:
                riserset.remove(i)
                continue
            glPushMatrix()
            glTranslatef(basex[bases[i]],ypos,-30.0)
            glCallList(basedl[bases[i]])
            glPopMatrix()

        for base in ('adenine','guanine','cytosine','thymine'):
            glPushMatrix()
            glTranslatef(basex[base],55.0,-20.0)
            glCallList(basedl[base])
            glPopMatrix()

        glDisable(GL_LIGHTING)

        for msg in list(msgset):
            if timer >= msg.expiration:
                msgset.remove(msg)
                msg.cleanup()
                continue
            msg.draw()

        liferatio = life[0]/maxlife
        if liferatio > 0.67:
            glColor3f(0.0,1.0,0.0)
        elif liferatio > 0.33:
            glColor3f(1.0,1.0,0.0)
        else:
            glColor3f(1.0,0.0,0.0)
        glBegin(GL_QUADS)
        glVertex3f(66.0,69.0,-1.0)    
        glVertex3f(66.0+liferatio*20.0,69.0,-1.0)    
        glVertex3f(66.0+liferatio*20.0,71.0,-1.0)    
        glVertex3f(66.0,71.0,-1.0)
        glEnd()
        glColor3f(0.0,0.0,0.0)
        glBegin(GL_LINE_LOOP)
        glVertex3f(66.0,69.0,-0.99)    
        glVertex3f(86.0,69.0,-0.99)    
        glVertex3f(86.0,71.0,-0.99)    
        glVertex3f(66.0,71.0,-0.99)
        glEnd()

        glEnable(GL_LIGHTING)

        if not paused:
            timer += 1

        clock.tick(30)
        pygame.display.flip()


def scoreboard(score,hits,nbases):
    if score < 0:
        messages = (
            Message(50.0,60.0,"FAIL",ttfont,(255,255,255),0),
            Message(50.0,35.0,"Your host has cancer now.",
                    opfont,(255,255,255),0),
            Message(50.0,25.0,"I hope you're happy.",
                    opfont,(255,255,255),0),
            )
        glClearColor(0.0,0.0,0.0,1.0)
    else:
        ratio = hits/nbases
        if ratio >= 1.0:
            reward1 = "YOU GOT THEM"
            reward2 = "ALL!!!"
        elif ratio >= 0.9:
            reward1 = "GREAT JOB"
            reward2 = None
        elif ratio >= 0.8:
            reward1 = "GOOD WORK"
            reward2 = None
        elif ratio >= 0.7:
            reward1 = "SATISFACTORY"
            reward2 = None
        elif ratio >= 0.6:
            reward1 = "YOU COULD DO"
            reward2 = "BETTER"
        else:
            reward1 = "BARELY ALIVE"
            reward2 = None
        messages = set((
            Message(50.0,40.0,"Score: %d"%score,opfont,(0,0,0),0),
            Message(50.0,25.0,
                    "Hit Ratio: %.3g (%d/%d)"%(ratio,hits,nbases),
                    opfont,(0,0,0),0),
            ))
        if reward2 is not None:
            messages.update((
                Message(50.0,65.0,reward1,ttfont,(0,0,0),0),
                Message(50.0,55.0,reward2,ttfont,(0,0,0),0),
                ))
        else:
            messages.add(Message(50.0,60.0,reward1,ttfont,(0,0,0),0))
        glClearColor(0.5,0.5,1.0,1.0)


    timer = 0
    while True:
        if timer > 240:
            for msg in messages:
                msg.cleanup()
            glClearColor(0.5,0.5,1.0,1.0)
            return

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                raise SystemExit
            elif event.type == pygame.KEYDOWN:
                if event.key in (pygame.K_q,pygame.K_ESCAPE):
                    raise SystemExit
                elif event.key in (pygame.K_KP_ENTER,pygame.K_SPACE,
                                   pygame.K_RETURN):
                    for msg in messages:
                        msg.cleanup()
                    glClearColor(0.5,0.5,1.0,1.0)
                    return

                      
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()

        glDisable(GL_LIGHTING)

        for msg in messages:
            msg.draw()

        glEnable(GL_LIGHTING)

        timer += 1
        clock.tick(30)
        pygame.display.flip()

